BufInsertFormatted

Struct BufInsertFormatted 

pub struct BufInsertFormatted { /* private fields */ }
A wrapper around InsertFormatted which buffers writes.

Implementations

Source

impl BufInsertFormatted

pub fn buf_len(&self) -> usize

Return the number of buffered bytes.

pub fn capacity(&self) -> usize

Return the current capacity of the buffer.

Note: Size is Not Constant This may be smaller than the original capacity if part of the buffer is still being used by the connection.

This may be larger if a call to Self::write_buffered() caused the buffer to expand.

pub fn write_buffered(&mut self, data: &[u8])

Write data to the buffer without waiting for it to be flushed.

May cause the buffer to resize to fit the data.

pub async fn write(&mut self, data: &[u8]) -> Result<usize>

Write some data to the buffer, flushing first if it is already full.

Returns the number of bytes written, which may be less than data.len() if the remaining capacity was smaller.

Cancel-safe. Until this returns Ok(n), the contents of data are not yet written to the buffer.

pub async fn flush(&mut self) -> Result<()>

Flush the buffer to the server as a single chunk.

If compression is enabled, the full buffer will be compressed.

pub async fn end(&mut self) -> Result<()>

Flushes the buffer, then calls InsertFormatted::end().

Cancel-safe.