pub struct BytesCursor { /* private fields */ }Expand description
A cursor over raw bytes of the response returned by Query::fetch_bytes.
Unlike RowCursor which emits rows deserialized as structures from
RowBinary, this cursor emits raw bytes without deserialization.
Integration
Additionally to BytesCursor::next and BytesCursor::collect,
this cursor implements:
AsyncReadandAsyncBufReadfortokio-based ecosystem.futures_util::Stream,futures_util::AsyncReadandfutures_util::AsyncBufReadforfutures-based ecosystem. (requires thefutures03feature)
For instance, if the requested format emits each row on a newline
(e.g. JSONEachRow, CSV, TSV, etc.), the cursor can be read line by
line using AsyncBufReadExt::lines. Note that this method
produces a new String for each line, so it’s not the most performant way
to iterate.
Note: methods of these traits use std::io::Error for errors.
To get an original error from this crate, use From conversion.
Implementations
Sourceimpl BytesCursor
impl BytesCursor
Sourcepub fn received_bytes(&self) -> u64
pub fn received_bytes(&self) -> u64
Returns the total size in bytes received from the CH server since the cursor was created.
This method counts only size without HTTP headers for now. It can be changed in the future without notice.
Sourcepub fn decoded_bytes(&self) -> u64
pub fn decoded_bytes(&self) -> u64
Returns the total size in bytes decompressed since the cursor was created.